import { CommonModule } from '@angular/common';
import {
  Component,
  Input,
  OnInit,
  ViewChild,
} from '@angular/core';
import {
  FormBuilder,
  FormControl,
  FormGroup,
  ReactiveFormsModule,
  Validators,
} from '@angular/forms';
import { forkJoin, of, Subscription } from 'rxjs';
import {
  catchError,
  debounceTime,
  distinctUntilChanged,
  switchMap,
} from 'rxjs/operators';
import { AutocompleteService } from '../../services/auto-complete/auto-complete.service';
import { ExifService } from '../../services/exif/exif.service';
import { ImageService } from '../../services/image/image.service';
import { MapReloadService } from '../../services/map-reload/map-reload.service';
import { ModalService } from '../../services/modal/modal.service';
import { PinService } from '../../services/pin/pin.service';
import { DragDropComponent } from '../drag-drop/drag-drop.component';

@Component({
  selector: 'app-add-pin-popup',
  standalone: true,
  imports: [ReactiveFormsModule, CommonModule, DragDropComponent],
  templateUrl: './add-pin-popup.component.html',
})
export class AddPinPopupComponent implements OnInit {
  @ViewChild(DragDropComponent) dragDropComponent!: DragDropComponent;
  form: FormGroup;
  suggestions: any[] = [];
  inputFocused: boolean = false;
  @Input() isHomePage: boolean = false;
  files: File[] = [];
  isPinModalOpen: boolean = false;
  modalId: string = 'add-pin-modal';
  private modalSub!: Subscription;
  uploadError: string = '';

  constructor(
    private fb: FormBuilder,
    private autocompleteService: AutocompleteService,
    private pinService: PinService,
    private exifService: ExifService,
    private modalService: ModalService,
    private mapReloadService: MapReloadService,
    private imageService: ImageService
  ) {
    this.form = this.fb.group({
      title: new FormControl('', [Validators.required, Validators.minLength(3)]),
      description: new FormControl('', [Validators.required, Validators.minLength(3)]),
      location: new FormControl('', [Validators.required]),
      complete_address: new FormControl('', [Validators.required]),
      coordinates: new FormControl<number[]>([]),
      files: new FormControl([]),
      date: new FormControl(''),
    });
  }

    onFocus(): void {
    this.inputFocused = true;
  }

  onBlur(): void {
    setTimeout(() => {
      this.inputFocused = false; // Désactiver le focus après un petit délai pour permettre un clic sur la liste
    }, 200);
  }

  ngOnInit(): void {
    this.modalSub = this.modalService
      .getModalState(this.modalId)
      .subscribe((open) => {
        this.isPinModalOpen = open;

        if (open) {
          const images = this.modalService.getImageFiles().getValue();
          if (images && images.length > 0) {
            this.files = images;
            this.form.patchValue({ files: images });
            // Convertir les fichiers en FileList pour le composant drag-drop
            const dataTransfer = new DataTransfer();
            images.forEach((file) => dataTransfer.items.add(file));
            const fileList = dataTransfer.files;
            if (this.dragDropComponent) {
              this.dragDropComponent.updateFileNamesFromFileList(fileList);
            }
          }

          // Récupérer les données pré-remplies du formulaire
          const formData = this.modalService.getFormData().getValue();
          if (formData) {
            this.form.patchValue(formData);
          }
        }
      });

    this.form
      .get('location')
      ?.valueChanges.pipe(
        debounceTime(200), // Attendre 200ms après la dernière frappe
        distinctUntilChanged(), // Ignorer si la nouvelle valeur est la même que la précédente
        switchMap((query) => {
          if(query === null) {
            return of([]);
          }
          const trimmedQuery = query.trim();
          if (trimmedQuery.length > 2) {
            return this.autocompleteService.getAddressSuggestions(trimmedQuery);
          }
          return of([]);
        }),
        catchError((error) => {
          console.error('Error fetching suggestions:', error);
          return of([]);
        })
      )
      .subscribe((data) => {
        this.suggestions = data;
      });
  }

  selectSuggestion(suggestion: any): void {
    const locationControl = this.form.get('location');
    if (locationControl instanceof FormControl) {
      locationControl.setValue(suggestion.display_name);
      this.form.get('complete_address')?.setValue(suggestion.display_name);
      this.form.get('coordinates')?.setValue([suggestion.lat, suggestion.lon]);
    }
    this.suggestions = [];
  }

  async onFilesReceived(files: FileList): Promise<void> {
    // Ajouter les nouveaux fichiers à la liste existante
    this.files = [...this.files, ...Array.from(files)];
    this.uploadError = ''; // Réinitialiser l'erreur

    if (this.dragDropComponent) {
      this.dragDropComponent.updateFileNamesFromFileList(files);
    } else {
      console.warn('AddPinPopupComponent - dragDropComponent not available');
    }

    // Ne traiter que la première photo pour les métadonnées EXIF
    if (files.length > 0) {
      try {
        const data = await this.exifService.getLocation(files[0]);
        if (data.latitude !== undefined && data.longitude !== undefined) {
          this.autocompleteService.getAddressFromCoordinates(data.latitude, data.longitude).subscribe((address) => {
            if (address) {
              this.form.get('location')?.setValue(address.display_name);
              this.form.get('complete_address')?.setValue(address.display_name);
              this.form.get('coordinates')?.setValue([data.latitude, data.longitude]);
            }
          });
        }
      } catch (error) {
        console.error(
          'AddPinPopupComponent - Error processing EXIF data:',
          error
        );
      }
    }
  }

  getFileNames(): string[] {
    return this.files.map((file) => file.name);
  }

  ngOnDestroy() {
    this.modalSub.unsubscribe();
  }

  submitForm(): void {
    // Marquer tous les champs comme touched pour afficher les erreurs
    Object.keys(this.form.controls).forEach(key => {
      const control = this.form.get(key);
      control?.markAsTouched();
    });

    if (this.form.valid) {
      const uploadObservables = this.files.map((file) => {
        if(file.size === 0) {
          this.uploadError = file.name + ' : ' + 'Image vide';
          return of(null);
        }
        return this.imageService.postImage(file).pipe(
          catchError(error => {
            this.uploadError = file.name + ' : ' + error.error.detail || 'Erreur lors de l\'upload de l\'image';
            if (this.dragDropComponent) {
              this.dragDropComponent.errorMessage = this.uploadError;
            }
            return of(null);
          })
        )
      });

      forkJoin(uploadObservables).subscribe((responses) => {
        // Vérifier si toutes les réponses sont valides
        if (responses.some(response => response === null)) {
          return; // Ne pas continuer si une erreur s'est produite
        }

        this.files = responses.map((res: any) => res.id);

        const coordinates = this.form.get('coordinates')?.value;
        const pinData = {
          ...this.form.value,
          files: this.files,
          date: this.form.get('date')?.value || null,
          location: coordinates || [0, 0], // Utiliser les coordonnées pour location
          complete_address: this.form.get('complete_address')?.value || this.form.get('location')?.value,
        };

        // Supprimer le champ coordinates qui n'est pas dans le modèle Pin
        delete pinData.coordinates;

        console.log('Files : ' + JSON.stringify(this.files));
        console.log('Pin Data : ' + JSON.stringify(pinData));

        this.pinService.addPin(pinData)?.subscribe(() => {
          this.mapReloadService.requestReload(); // Demander le rechargement de la carte
          this.closePinModal();
        });
      });
    } else {
      console.error('Le formulaire est invalide');
    }
  }

  openPinModal() {
    this.modalService.openModal(this.modalId);
  }

  closePinModal() {
    this.modalService.closeModal(this.modalId);
    this.form.reset();
    this.files = [];
    this.uploadError = '';
    if (this.dragDropComponent) {
      this.dragDropComponent.updateFileNamesFromFileList(new DataTransfer().files);
      this.dragDropComponent.errorMessage = '';
    }
  }

  getImagePreview(file: File): string {
    return URL.createObjectURL(file);
  }

  removeFile(fileName: string): void {
    const index = this.files.findIndex((file) => file.name === fileName);
    if (index > -1) {
      this.files.splice(index, 1);
      this.uploadError = ''; // Réinitialiser l'erreur lors de la suppression d'un fichier
      if (this.dragDropComponent) {
        this.dragDropComponent.errorMessage = '';
      }

      // Mettre à jour le form control
      const dataTransfer = new DataTransfer();
      this.files.forEach((file) => dataTransfer.items.add(file));
      this.form.patchValue({ files: dataTransfer.files });
    }
  }
}
