import { CommonModule } from '@angular/common';
import {
  Component,
  EventEmitter,
  Input,
  OnDestroy,
  OnInit,
  ViewChild,
} from '@angular/core';
import {
  FormBuilder,
  FormControl,
  FormGroup,
  ReactiveFormsModule,
  Validators,
} from '@angular/forms';
import { NavigationEnd, Router } from '@angular/router';
import { forkJoin, of, Subscription } from 'rxjs';
import {
  catchError,
  debounceTime,
  distinctUntilChanged,
  filter,
  switchMap,
  take,
} from 'rxjs/operators';
import { Pin } from '../../model/Pin';
import { AutocompleteService } from '../../services/auto-complete/auto-complete.service';
import { ExifService } from '../../services/exif/exif.service';
import { ImageService } from '../../services/image/image.service';
import { MapReloadService } from '../../services/map-reload/map-reload.service';
import { ModalService } from '../../services/modal/modal.service';
import { PinService } from '../../services/pin/pin.service';
import { DragDropComponent } from '../drag-drop/drag-drop.component';

@Component({
  selector: 'app-edit-pin-popup',
  standalone: true,
  imports: [ReactiveFormsModule, CommonModule, DragDropComponent],
  templateUrl: './edit-pin-popup.component.html',
})
export class EditPinPopupComponent implements OnInit, OnDestroy {
  @Input() isHomePage: boolean = false;
  @Input() pin!: Pin;
  @Input() pinId!: string;
  
  @Input() pinOpened!: EventEmitter<void>; 

  @ViewChild(DragDropComponent) dragDropComponent!: DragDropComponent;
  
  private modalOpenSubscription!: Subscription;

  form!: FormGroup;
  suggestions: any[] = [];
  inputFocused: boolean = false;
  files: File[] = [];
  isPinModalOpen: boolean = false;
  uploadError: string = '';
  modalId: string = '';

  constructor(
    private fb: FormBuilder,
    private autocompleteService: AutocompleteService,
    private pinService: PinService,
    private exifService: ExifService,
    private modalService: ModalService,
    private mapReloadService: MapReloadService,
    private imageService: ImageService
  ) {
    // Initialiser le formulaire avec des valeurs par défaut
    this.form = this.fb.group({
      title: new FormControl('', [Validators.required, Validators.minLength(3)]),
      description: new FormControl('', [Validators.required, Validators.minLength(3)]),
      location: new FormControl('', [Validators.required]),
      complete_address: new FormControl('', [Validators.required]),
      coordinates: new FormControl<number[]>([]),
      files: new FormControl(null),
      date: new FormControl(''),
    });
  }

  onFocus(): void {
    this.inputFocused = true;
  }

  onBlur(): void {
    setTimeout(() => {
      this.inputFocused = false; // Désactiver le focus après un petit délai pour permettre un clic sur la liste
    }, 200);
  }

  ngOnInit(): void {
    this.modalId = 'edit-pin-popup-' + this.pinId;
    
    // S'abonner aux changements d'état du modal
    this.modalOpenSubscription = this.modalService
      .getModalState(this.modalId)
      .subscribe((state) => {
        this.isPinModalOpen = state;
      });

    // S'abonner aux événements de navigation du router
    this.pinOpened.subscribe(() => {
      this.moveModalToBody();
    });

    // Configuration de l'autocomplétion pour le champ d'adresse
    this.form
      .get('location')
      ?.valueChanges.pipe(
        debounceTime(300), // Attendre 300ms après la dernière frappe
        distinctUntilChanged(), // Ignorer si la nouvelle valeur est la même que la précédente
        switchMap((query) => {
          // Vérifier que query est une chaîne de caractères          
          if (typeof query !== 'string') {
            return of([]);
          }

          const trimmedQuery = query.trim();
          if (trimmedQuery.length > 2) {
            return this.autocompleteService.getAddressSuggestions(trimmedQuery);
          }
          return of([]);
        }),
        catchError((error) => {
          console.error('Error fetching suggestions:', error);
          return of([]);
        })
      )
      .subscribe((data) => {
        this.suggestions = data;
      });
  }

  ngOnDestroy() {
    // Nettoyage des abonnements pour éviter les fuites de mémoire
    if (this.modalOpenSubscription) {
      this.modalOpenSubscription.unsubscribe();
    }
  }

  // Méthode dédiée pour déplacer le modal vers le body
  private moveModalToBody(): void {
    const modal = document.getElementById(this.modalId);
    if (modal && modal.parentElement !== document.body) {
      document.body.appendChild(modal);
    }
  }

  selectSuggestion(suggestion: any): void {
    const locationControl = this.form.get('location');
    if (locationControl instanceof FormControl) {
      locationControl.setValue(suggestion.display_name);
      this.form.get('complete_address')?.setValue(suggestion.display_name);
      this.form.get('coordinates')?.setValue([suggestion.lat, suggestion.lon]);
    }
    this.suggestions = [];
  }

  async onFilesReceived(files: FileList): Promise<void> {
    // Ajouter les nouveaux fichiers à la liste existante
    this.files = [...this.files, ...Array.from(files).map((file) => file)];
    this.uploadError = ''; // Réinitialiser l'erreur

    if (this.dragDropComponent) {
      this.dragDropComponent.updateFileNamesFromFileList(files);
    } else {
      console.warn('EditPinPopupComponent - dragDropComponent not available');
    }

    // Ne traiter que la première photo pour les métadonnées EXIF
    if (files.length > 0) {
      try {
        const data = await this.exifService.getLocation(files[0]);
        if (data.latitude !== undefined && data.longitude !== undefined) {
          this.autocompleteService.getAddressFromCoordinates(data.latitude, data.longitude).subscribe((address) => {
            if (address) {
              this.form.get('location')?.setValue(address.display_name);
              this.form.get('complete_address')?.setValue(address.display_name);
              this.form.get('coordinates')?.setValue([data.latitude, data.longitude]);
            }
          });
        }
      } catch (error) {
        console.error(
          'EditPinPopupComponent - Error processing EXIF data:',
          error
        );
      }
    }
  }

  submitForm(): void {
    // Marquer tous les champs comme touched pour afficher les erreurs
    Object.keys(this.form.controls).forEach(key => {
      const control = this.form.get(key);
      control?.markAsTouched();
    });

    if (this.form.valid) {
      const uploadObservables = this.files.map((file) => {
        if(file.size === 0) {
          if(file.name.includes("|")) {
            return of({id: file.name.split("|")[1]});
          } else {
            this.uploadError = file.name + ' : ' + 'Image vide';
            return of(null);
          }
        }
        return this.imageService.postImage(file).pipe(
          catchError(error => {
            this.uploadError = file.name + ' : ' + error.error.detail || 'Erreur lors de l\'upload de l\'image';
            if (this.dragDropComponent) {
              this.dragDropComponent.errorMessage = this.uploadError;
            }
            return of(null);
          })
        )
      });

      forkJoin(uploadObservables).subscribe((responses) => {
        // Vérifier si toutes les réponses sont valides
        if (responses.some(response => response === null)) {
          return; // Ne pas continuer si une erreur s'est produite
        }

        this.files = responses.map((res: any) => res.id);

        const coordinates = this.form.get('coordinates')?.value;
        const pinData = {
          ...this.form.value,
          files: this.files,
          date: this.form.get('date')?.value || null,
          location: coordinates || [0, 0],
          complete_address: this.form.get('complete_address')?.value || this.form.get('location')?.value,
        };

        delete pinData.coordinates;

        this.pinService.updatePin(this.pin.id, pinData)?.subscribe(() => {
          this.mapReloadService.requestReload();
          this.closePinModal();
        });
      });
    } else {
      console.error('Le formulaire est invalide');
    }
  }

  openPinModal() {
    // Initialiser le formulaire avec les valeurs de base
    this.form.patchValue({
      title: this.pin?.title || '',
      description: this.pin?.description || '',
      location: this.pin?.complete_address || '',
      complete_address: this.pin?.complete_address || '',
      coordinates: this.pin?.location || [],
      files: this.pin?.files || [],
      date: this.pin?.date
        ? new Date(this.pin.date).toISOString().split('T')[0]
        : '',
    });
    
    this.pin.files.forEach((file) => {
      this.imageService.getImageMetadata(file).subscribe((metadata) => {
        this.files.push(new File([], metadata.metadata.original_filename + "|" + file.toString(), { type: metadata.metadata.content_type }));
      });
    });
    
    this.modalService.openModal(this.modalId);
  }

  closePinModal() {
    this.files = [];
    this.modalService.closeModal(this.modalId);
  }

  removeFile(fileName: string): void {
    const index = this.files.findIndex((file) => file.name === fileName || file.name.split("|")[0] === fileName);
    if (index > -1) {
      this.files.splice(index, 1);
      this.uploadError = ''; // Réinitialiser l'erreur lors de la suppression d'un fichier
      if (this.dragDropComponent) {
        this.dragDropComponent.errorMessage = '';
      }

      // Mettre à jour le form control
      const dataTransfer = new DataTransfer();
      this.files.forEach((file) => dataTransfer.items.add(file as File));
      this.form.patchValue({ files: dataTransfer.files });
    }
  }

  getFileNames(): string[] {
    return this.files.map(file => {
      if(typeof file === 'string') {
        return file;
      } else {
        return file.name.split("|")[0];
      }
    });
  }
}
