import { Injectable } from '@angular/core';
import { HttpClient, HttpHeaders, HttpParams } from '@angular/common/http';
import { BehaviorSubject, Observable, tap } from 'rxjs';
import { environment } from '../../../environment';
import { LocalStorageService } from '../local-storage/local-storage.service';
import { AuthResponse } from '../../model/AuthResponse';

@Injectable({
  providedIn: 'root'
})
export class AuthService {
  private isAdminSubject = new BehaviorSubject<boolean>(false);
  isAdmin$ = this.isAdminSubject.asObservable();
  username$ = new BehaviorSubject<string>('');
  isLoggedIn$ = new BehaviorSubject<boolean>(false);

  constructor(private http: HttpClient, private localStorageService: LocalStorageService) {
    const token = this.localStorageService.getToken();
    if (token) {
      this.isAdminSubject.next(this.localStorageService.getIsAdmin() === 'true');
      this.username$.next(this.localStorageService.getUsername() || '');
      this.isLoggedIn$.next(true);
    }
  }

  login(username: string, password: string): Observable<AuthResponse> {
    const payload = new HttpParams()
      .set('username', username)
      .set('password', password);

    return this.http.post<AuthResponse>(`${environment.apiURL}/login`, payload).pipe(
      tap(response => {
        this.localStorageService.setToken(response.access_token);
        this.localStorageService.setIsAdmin(response.is_admin);
        this.localStorageService.setUsername(username);
        this.isAdminSubject.next(response.is_admin);
        this.username$.next(username);
        this.isLoggedIn$.next(true);
      })
    );
  }

  logout(): void {
    this.localStorageService.removeToken();
    this.localStorageService.removeIsAdmin();
    this.isAdminSubject.next(false);
    this.username$.next('');
    this.isLoggedIn$.next(false);
  }

  register(username: string, password: string): Observable<AuthResponse> {
    return this.http.post<AuthResponse>(`${environment.apiURL}/register`, { username, password }).pipe(
      tap(response => {
        this.localStorageService.setToken(response.access_token);
        this.localStorageService.setIsAdmin(response.is_admin);
        this.localStorageService.setUsername(username);
        this.isAdminSubject.next(response.is_admin);
        this.username$.next(username);
        this.isLoggedIn$.next(true);
      })
    );
  }

  isAdmin(): boolean {
    return this.isAdminSubject.value || false;
  }

  getAuthHeaders(): HttpHeaders {
    const token = this.localStorageService.getToken();
    return new HttpHeaders().set('Authorization', `Bearer ${token}`);
  }

  getUsername(): string {
    return this.username$.value || '';
  }

  isLoggedIn(): boolean {
    return this.isLoggedIn$.value || false;
  }
} 