import { Injectable } from '@angular/core';
import * as exifr from 'exifr';

@Injectable({
  providedIn: 'root',
})
export class ExifService {
  private getExifData(file: File): Promise<any> {
    return exifr.parse(file);
  }

  async getAllExifData(file: File): Promise<any> {
    try {
      return await this.getExifData(file);
    } catch (error) {
      console.error('Error reading EXIF data:', error);
      return;
    }
  }

  async getOrientation(file: File): Promise<number | undefined> {
    try {
      const exifData = await this.getExifData(file);
      return exifData.Orientation;
    } catch (error) {
      console.error('Error reading EXIF data:', error);
      return undefined;
    }
  }

  async getDeviceModel(file: File): Promise<string | undefined> {
    try {
      const exifData = await this.getExifData(file);
      return exifData.Model;
    } catch (error) {
      console.error('Error reading EXIF data:', error);
      return undefined;
    }
  }

  async getLocation(
    file: File
  ): Promise<{ latitude?: number; longitude?: number }> {
    try {
      const exifData = await this.getExifData(file);
      return {
        latitude: exifData.GPSLatitude
          ? this.convertToDecimal(exifData.GPSLatitude, exifData.GPSLatitudeRef)
          : undefined,
        longitude: exifData.GPSLongitude
          ? this.convertToDecimal(
              exifData.GPSLongitude,
              exifData.GPSLongitudeRef
            )
          : undefined,
      };
    } catch (error) {
      console.error('Error reading EXIF data:', error);
      return {};
    }
  }

  async getDateTime(file: File): Promise<string | boolean> {
    try {
      const exifData = await this.getExifData(file);
      return exifData.DateTime;
    } catch (error) {
      console.error('Error reading EXIF data:', error);
      return false;
    }
  }

  private convertToDecimal(coordinate: number[], direction: string): number {
    if (!coordinate || coordinate.length !== 3) return NaN;
    const decimal = coordinate[0] + coordinate[1] / 60 + coordinate[2] / 3600;
    return direction === 'S' || direction === 'W' ? -decimal : decimal;
  }
}
